@@ -37,6 +37,7 @@
 #include <Lum/Menu.h>
 #include <Lum/Panel.h>
 #include <Lum/Text.h>
+#include <Lum/Toggle.h>
 
 #include <Lum/Model/Action.h>
 #include <Lum/Model/String.h>
@@ -120,21 +121,16 @@ public:
 class MainDialog : public Lum::Dialog
 {
 private:
-  Lum::Model::ActionRef  generateAction;
-  Lum::Model::ActionRef  solveAction;
-  Lum::Model::ActionRef  statAction;
-  Lum::Model::ActionRef  helpAction;
-  Lum::Model::ActionRef  aboutAction;
-  Lum::Model::StringRef  minimumPoints;
-  GameArea               *gameArea;
-  Lum::Text              *filledFields;
-  Lum::Text              *directFills;
-  Lum::Text              *crossHatch;
-  Lum::Text              *singleCandidate;
-  Lum::Text              *singleSquare;
-  Lum::Text              *guesses;
-  Lum::Text              *rating;
-  GenerateTask           generateTask;
+  Lum::Model::ActionRef   generateAction;
+  Lum::Model::ActionRef   solveAction;
+  Lum::Model::ActionRef   statAction;
+  Lum::Model::ActionRef   helpAction;
+  Lum::Model::ActionRef   aboutAction;
+  Lum::Model::StringRef   minimumPoints;
+  GameArea                *gameArea;
+  Lum::Text               *rating;
+  GenerateTask            generateTask;
+  Lum::Model::BooleanRef  keypad[Sudoku::dimension];
 
 public:
   MainDialog()
@@ -145,13 +141,13 @@ public:
     aboutAction(new Lum::Model::Action),
     minimumPoints(new Lum::Model::String()),
     gameArea(new GameArea()),
-    directFills(NULL),
-    crossHatch(NULL),
-    singleCandidate(NULL),
-    singleSquare(NULL),
-    guesses(NULL),
     rating(NULL)
   {
+    for (size_t i=0; i<Sudoku::dimension; i++) {
+      keypad[i]=new Lum::Model::Boolean(false);
+      Observe(keypad[i]);
+    }
+
     generateTask.SetParent(this);
     generateTask.SetCaption(_(L"PROGRESS_TITLE",L"Generating riddle..."));
 
@@ -167,6 +163,7 @@ public:
     Observe(generateTask.GetAbortedAction());
     Observe(generateTask.GetFinishedAction());
     gameArea->GetFinishedModel()->Off(); // Switch it off initialy and on after config reading!
+    Observe(gameArea->GetFocusChangedAction());
     Observe(gameArea->GetFinishedModel());
   }
 
@@ -189,18 +186,6 @@ public:
 
     vert=Lum::VPanel::Create(false,true);
 
-    array=new Lum::Array();
-    array->SetFlex(true,false);
-    array->SetVerticalCount(1);
-    array->SetSpace(true,false);
-
-    array->Add(Lum::Button::Create(_(L"GENERATE",L"_Generate"),generateAction,true,false));
-    array->Add(Lum::Button::Create(_(L"SOLVE",L"_Solve"),solveAction,true,false));
-
-    vert->Add(array);
-
-    vert->AddSpace();
-
     label=Lum::Label::Create(false,true);
 
     pointsModel=new Lum::Model::StringTable();
@@ -215,49 +200,7 @@ public:
     combo->SetModel(minimumPoints);
     combo->SetTableModel(pointsModel);
     dynamic_cast<Lum::TextEditCombo*>(combo)->GetString()->SetAlignment(Lum::String::right);
-    label->AddLabel(_(L"POINTS_MIN",L"min. Points:"),combo);
-
-    filledFields=new Lum::Text();
-    filledFields->SetFlex(true,false);
-    filledFields->SetWidth(Lum::Base::Size::stdCharWidth,2);
-    filledFields->SetAlignment(Lum::Text::right);
-    filledFields->SetText(L"0");
-    label->AddLabel(_(L"POINTS_FILLED_FIELDS",L"Filled fields:"),filledFields);
-
-    directFills=new Lum::Text();
-    directFills->SetFlex(true,false);
-    directFills->SetWidth(Lum::Base::Size::stdCharWidth,2);
-    directFills->SetAlignment(Lum::Text::right);
-    directFills->SetText(L"0");
-    label->AddLabel(_(L"POINTS_DIRECT_FILLS",L"Direct fills:"),directFills);
-
-    crossHatch=new Lum::Text();
-    crossHatch->SetFlex(true,false);
-    crossHatch->SetWidth(Lum::Base::Size::stdCharWidth,2);
-    crossHatch->SetAlignment(Lum::Text::right);
-    crossHatch->SetText(L"0");
-    label->AddLabel(_(L"POINTS_CROSSHATCH",L"Cross hatch:"),crossHatch);
-
-    singleCandidate=new Lum::Text();
-    singleCandidate->SetFlex(true,false);
-    singleCandidate->SetWidth(Lum::Base::Size::stdCharWidth,2);
-    singleCandidate->SetAlignment(Lum::Text::right);
-    singleCandidate->SetText(L"0");
-    label->AddLabel(_(L"POINTS_SINGLE_CAND",L"Single candidate:"),singleCandidate);
-
-    singleSquare=new Lum::Text();
-    singleSquare->SetFlex(true,false);
-    singleSquare->SetWidth(Lum::Base::Size::stdCharWidth,2);
-    singleSquare->SetAlignment(Lum::Text::right);
-    singleSquare->SetText(L"0");
-    label->AddLabel(_(L"POINTS_SINGLE_SQUARE",L"Single square:"),singleSquare);
-
-    guesses=new Lum::Text();
-    guesses->SetFlex(true,false);
-    guesses->SetWidth(Lum::Base::Size::stdCharWidth,2);
-    guesses->SetAlignment(Lum::Text::right);
-    guesses->SetText(L"0");
-    label->AddLabel(_(L"POINTS_GUESS",L"Guesses:"),guesses);
+    label->AddLabel(_(L"POINTS_MIN",L"Min. Points:"),combo);
 
     rating=new Lum::Text();
     rating->SetFlex(true,false);
@@ -269,6 +212,35 @@ public:
 
     vert->Add(label);
 
+    vert->AddSpace(true);
+
+    if (Lum::OS::display->GetTheme()->RequestFingerFriendlyControls()) {
+      array=new Lum::Array();
+      array->SetFlex(true,true);
+      array->SetHorizontalCount(3);
+      array->SetSpace(true,false);
+
+      for (size_t i=0; i<Sudoku::dimension; i++) {
+        array->Add(Lum::Toggle::Create(Lum::Base::NumberToWString(i+1),keypad[i],true,true));
+      }
+
+      vert->Add(array);
+
+      vert->AddSpace(true);
+    }
+
+    array=new Lum::Array();
+    array->SetFlex(true,false);
+    array->SetHorizontalCount(1);
+    array->SetSpace(false,false);
+
+
+    array->Add(Lum::Button::Create(_(L"GAME_STAT",L"S_tatistics"),statAction,true,false));
+    array->Add(Lum::Button::Create(_(L"SOLVE",L"_Solve"),solveAction,true,false));
+    array->Add(Lum::Button::Create(_(L"GENERATE",L"_Generate"),generateAction,true,false));
+
+    vert->Add(array);
+
     horiz->Add(vert);
 
     SetMain(horiz);
@@ -313,6 +285,47 @@ public:
     generateTask.Start();
   }
 
+  void UpdateKeypad()
+  {
+    for (size_t i=0; i<Sudoku::dimension; i++) {
+      if (gameArea->GetFinishedModel()->Get() || !gameArea->IsFocusedCellEditable()) {
+        keypad[i]->Set(false);
+        keypad[i]->Disable();
+      }
+      else {
+        keypad[i]->Enable();
+      }
+    }
+
+    if (gameArea->GetFinishedModel()->Get() || !gameArea->IsFocusedCellEditable()) {
+      return;
+    }
+
+    size_t value=gameArea->GetFocusedCellValue();
+
+    for (size_t i=0; i<Sudoku::dimension; i++) {
+      keypad[i]->Set(value & (1 << i));
+    }
+  }
+
+  void UpdateCell()
+  {
+    if (gameArea->GetFinishedModel()->Get() || !gameArea->IsFocusedCellEditable()) {
+      // We ignore possible changes if there is no valid game
+      return;
+    }
+
+    size_t value=0;
+
+    for (size_t i=0; i<Sudoku::dimension; i++) {
+      if (keypad[i].Valid() && keypad[i]->Get()) {
+        value|=(1 << i);
+      }
+    }
+
+    gameArea->SetFocusedCellValue(value);
+  }
+
   void SetRiddle()
   {
     char value[20];
@@ -320,12 +333,6 @@ public:
     sprintf(value,"%.0f",generateTask.weight.rating);
 
     gameArea->SetModel(riddle,solution);
-    filledFields->SetText(Lum::Base::NumberToWString(generateTask.weight.filledFields));
-    directFills->SetText(Lum::Base::NumberToWString(generateTask.weight.directFills));
-    crossHatch->SetText(Lum::Base::NumberToWString(generateTask.weight.crossHatch));
-    singleCandidate->SetText(Lum::Base::NumberToWString(generateTask.weight.singleCandidate));
-    singleSquare->SetText(Lum::Base::NumberToWString(generateTask.weight.singleSquare));
-    guesses->SetText(Lum::Base::NumberToWString(generateTask.weight.guesses));
     rating->SetText(Lum::Base::StringToWString(value));
   }
 
@@ -346,22 +353,17 @@ public:
           gameArea->SetGame(*game);
 
           sprintf(value,"%.0f",weight.rating);
-
-          filledFields->SetText(Lum::Base::NumberToWString(weight.filledFields));
-          directFills->SetText(Lum::Base::NumberToWString(weight.directFills));
-          crossHatch->SetText(Lum::Base::NumberToWString(weight.crossHatch));
-          singleCandidate->SetText(Lum::Base::NumberToWString(weight.singleCandidate));
-          singleSquare->SetText(Lum::Base::NumberToWString(weight.singleSquare));
-          guesses->SetText(Lum::Base::NumberToWString(weight.guesses));
           rating->SetText(Lum::Base::StringToWString(value));
 
           delete game;
           game=NULL;
         }
       }
+
       if (gameArea->GetFinishedModel()->Get()) {
         solveAction->Disable();
       }
+
       gameArea->GetFinishedModel()->On(false);
     }
     else if (model==GetClosedAction() && GetClosedAction()->IsFinished()) {
@@ -398,10 +400,14 @@ public:
     }
     else if (model==statAction && statAction->IsFinished()) {
       Sudoku game;
+      Sudoku riddle;
+      Sudoku solution;
 
       gameArea->GetGame(game);
+      gameArea->GetRiddle(riddle);
+      gameArea->GetSolution(solution);
 
-      StatisticsDialog::Show(GetWindow(),game);
+      StatisticsDialog::Show(GetWindow(),game,riddle,solution);
     }
     else if (model==helpAction && helpAction->IsFinished()) {
       Lum::Dlg::Help::Show(this,
@@ -432,8 +438,18 @@ public:
                               L"the given riddle!\n"
                               L"Congratulations!"));
       solveAction->Disable();
+      UpdateKeypad();
+    }
+    else if (model==gameArea->GetFocusChangedAction() && gameArea->GetFocusChangedAction()->IsFinished()) {
+      UpdateKeypad();
+    }
+    else {
+      for (size_t i=0; i<Sudoku::dimension; i++) {
+        if (model==keypad[i]) {
+          UpdateCell();
+        }
+      }
     }
-
 
     Dialog::Resync(model,msg);
   }
