@@ -20,14 +20,17 @@
 #include "Statistics.h"
 
 #include <Lum/Base/L10N.h>
+#include <Lum/Base/String.h>
 
 #include <Lum/Model/DataStream.h>
 
 #include <Lum/ButtonRow.h>
 #include <Lum/Dialog.h>
 #include <Lum/Grid.h>
+#include <Lum/Label.h>
 #include <Lum/Panel.h>
 #include <Lum/PercentBar.h>
+#include <Lum/Tab.h>
 #include <Lum/Text.h>
 
 static Lum::Object* GetPercentBar(size_t count)
@@ -40,6 +43,7 @@ static Lum::Object* GetPercentBar(size_t count)
   stream->Set(0,(1.0*count)/9);
 
   bar=new Lum::PercentBar();
+  bar->SetFlex(true,true);
   bar->DisplayVertical();
   bar->SetHeight(Lum::Base::Size::stdCharHeight,5);
   bar->SetModel(stream);
@@ -59,8 +63,10 @@ static Lum::Object* GetText(const std::wstring& label)
   return text;
 }
 
-StatisticsDialog::StatisticsDialog(const Sudoku& game)
- : game(game)
+StatisticsDialog::StatisticsDialog(const Sudoku& game, const Sudoku& riddle, const Sudoku& solution)
+ : game(game),
+   riddle(riddle),
+   solution(solution)
 {
   SetExitAction(GetClosedAction());
 }
@@ -70,14 +76,19 @@ StatisticsDialog::~StatisticsDialog()
   // No code
 }
 
-void StatisticsDialog::PreInit()
+Lum::Object* StatisticsDialog::GetContent()
 {
-  Lum::Grid        *grid;
-  Lum::Panel       *vert;
+  Lum::Grid       *grid;
+  Lum::Label      *label;
+  Lum::Tab        *tab;
+  Sudoku:: Weight weight;
 
-  vert=new Lum::VPanel();
+  riddle.CalculateWeight(solution,weight);
+
+  tab=Lum::Tab::Create(true,true);
 
   grid=new Lum::Grid();
+  grid->SetFlex(true,true);
   grid->SetSpace(true,true);
   grid->SetSize(9,2);
   grid->SetEqualDimensions(true,false);
@@ -109,21 +120,53 @@ void StatisticsDialog::PreInit()
   grid->SetObject(6,1,GetText(L"7"));
   grid->SetObject(7,1,GetText(L"8"));
   grid->SetObject(8,1,GetText(L"9"));
-  vert->Add(grid);
+  tab->Add(L"Distribution",grid);
 
-  vert->AddSpace();
-  vert->Add(Lum::ButtonRow::CreateClose(GetClosedAction(),true,false));
+  label=Lum::Label::Create(true,true);
 
-  SetMain(vert);
+  label->AddLabel(_(L"POINTS_FILLED_FIELDS",L"Filled fields:"),
+                  Lum::Text::Create(Lum::Base::NumberToWString(weight.filledFields),
+                                    Lum::Text::right,
+                                    true,false));
+  label->AddLabel(_(L"POINTS_DIRECT_FILLS",L"Direct fills:"),
+                  Lum::Text::Create(Lum::Base::NumberToWString(weight.directFills),
+                                    Lum::Text::right,
+                                    true,false));
+  label->AddLabel(_(L"POINTS_CROSSHATCH",L"Cross hatch:"),
+                  Lum::Text::Create(Lum::Base::NumberToWString(weight.crossHatch),
+                                    Lum::Text::right,
+                                    true,false));
+  label->AddLabel(_(L"POINTS_SINGLE_CAND",L"Single candidate:"),
+                  Lum::Text::Create(Lum::Base::NumberToWString(weight.singleCandidate),
+                                    Lum::Text::right,
+                                    true,false));
+  label->AddLabel(_(L"POINTS_SINGLE_SQUARE",L"Single square:"),
+                  Lum::Text::Create(Lum::Base::NumberToWString(weight.singleSquare),
+                                    Lum::Text::right,
+                                    true,false));
+  label->AddLabel(_(L"POINTS_GUESS",L"Guesses:"),
+                  Lum::Text::Create(Lum::Base::NumberToWString(weight.guesses),
+                                    Lum::Text::right,
+                                    true,false));
 
-  Dialog::PreInit();
+  tab->Add(L"Rating",label);
+
+  return tab;
 }
 
-void StatisticsDialog::Show(Lum::OS::Window* parent, const Sudoku& game)
+void StatisticsDialog::GetActions(std::vector<Lum::Dlg::ActionInfo>& actions)
+{
+  Lum::Dlg::ActionDialog::CreateActionInfosClose(actions,GetClosedAction());
+}
+
+void StatisticsDialog::Show(Lum::OS::Window* parent,
+                            const Sudoku& game,
+                            const Sudoku& riddle,
+                            const Sudoku& solution)
 {
   StatisticsDialog *dialog;
 
-  dialog=new StatisticsDialog(game);
+  dialog=new StatisticsDialog(game,riddle,solution);
   dialog->SetParent(parent);
   dialog->SetTitle(L"Statistics...");
 
